@@ -1,7 +1,7 @@
 #-----------------------
 # @author: Tony Ribeiro
 # @created: 2019/04/15
-# @updated: 2021/01/15
+# @updated: 2021/01/26
 #
 # @desc: simple GULA implementation, the General Usage LFIT Algorithm.
 #   - INPUT: a set of pairs of discrete multi-valued states
@@ -16,11 +16,7 @@
 #       - about O( |observations| * |values| ^ (2 * |variables|) )
 #-----------------------
 
-#from utils import eprint
-#from rule import Rule
-#from logicProgram import LogicProgram
 from ..utils import eprint
-from ..objects.logicProgram import LogicProgram
 from ..objects.rule import Rule
 from ..algorithms.algorithm import Algorithm
 from ..datasets import StateTransitionsDataset
@@ -72,6 +68,15 @@ class GULA (Algorithm):
             or not all(isinstance(key, str) and isinstance(value, list) for key, value in targets_to_learn.items()) \
             or not all(isinstance(v, str) for key, value in targets_to_learn.items() for v in value):
             raise ValueError('targets_to_learn must be a dict of format {String: list of String}')
+        else:
+            for key, values in targets_to_learn.items():
+                targets_names = [var for var, vals in dataset.targets]
+                if key not in targets_names:
+                    raise ValueError('targets_to_learn keys must be dataset target variables')
+                var_id = targets_names.index(key)
+                for val in values:
+                    if val not in dataset.targets[var_id][1]:
+                        raise ValueError('targets_to_learn values must be in target variable domain')
 
         feature_domains = dataset.features
         target_domains = dataset.targets
@@ -141,9 +146,6 @@ class GULA (Algorithm):
                 eprint("\nStart learning of var=", var_id+1,"/", len(target_domains), ", val=", val_id+1, "/", len(target_domains[var_id][1]))
                 rules += GULA.fit_var_val(feature_domains, var_id, val_id, negatives, positives) #variables, values, var, val, negatives, program)#, partial_heuristic)
 
-        # Instanciate output logic program
-        #output = LogicProgram(feature_domains, target_domains, rules)
-
         return rules
 
 
