@@ -1,7 +1,7 @@
 #-----------------------
 # @author: Tony Ribeiro
 # @created: 2019/03/20
-# @updated: 2019/05/03
+# @updated: 2021/01/26
 #
 # @desc: simple approximated version of GULA implementation.
 #    - extract patern from pair of interpretation of transitions
@@ -10,8 +10,8 @@
 
 from ..utils import eprint
 from ..objects.rule import Rule
-from ..objects.logicProgram import LogicProgram
-from ..algorithms.algorithm import Algorithm
+from ..algorithms import Algorithm
+from ..datasets import StateTransitionsDataset
 
 import csv
 
@@ -21,56 +21,74 @@ class PRIDE (Algorithm):
     Learn logic rules that explain state transitions of a discrete dynamic system.
     """
 
-    @staticmethod
-    def fit(data, feature_domains, target_domains): # variables, values, transitions):
+    def fit(dataset, targets_to_learn=None): #variables, values, transitions, conclusion_values=None, program=None): #, partial_heuristic=False):
         """
-        Preprocess transitions and learn rules for all observed variables/values.
+        Preprocess transitions and learn rules for all given features/targets variables/values.
 
         Args:
-            data: list of tuple (list of int, list of int)
+            dataset: pylfit.datasets.StateTransitionsDataset
                 state transitions of a the system
-            feature_domains: list of (String, list of String)
-                feature variables of the system and their values
-            target_domains: list of (String, list of String)
-                target variables of the system and their values
+            targets_to_learn: dict of {String: list of String}
+                target variables values of the dataset for wich we want to learn rules.
+                If not given, all targets values will be learned.
 
         Returns:
-            LogicProgram
-                A logic program whose rules:
-                    - are minimals
-                    - explain/reproduce all the input transitions
+            list of pylfit.objects.Rule
+                A set of DMVLP rules that is:
+                    - correct: explain/reproduce all the transitions of the dataset.
+                    - optimal: all rules are minimals
         """
+
         #eprint("Start PRIDE learning...")
 
-        # Nothing to learn
-        if len(data) == 0:
-            return LogicProgram(feature_domains, target_domains, [])
+        # Parameters checking
+        if not isinstance(dataset, StateTransitionsDataset):
+            raise ValueError('Dataset type not supported, PRIDE expect ' + str(StateTransitionsDataset.__name__))
 
+        if targets_to_learn is None:
+            targets_to_learn = dict()
+            for a, b in dataset.targets:
+                targets_to_learn[a] = b
+        elif not isinstance(targets_to_learn, dict) \
+            or not all(isinstance(key, str) and isinstance(value, list) for key, value in targets_to_learn.items()) \
+            or not all(isinstance(v, str) for key, value in targets_to_learn.items() for v in value):
+            raise ValueError('targets_to_learn must be a dict of format {String: list of String}')
+        else:
+            for key, values in targets_to_learn.items():
+                targets_names = [var for var, vals in dataset.targets]
+                if key not in targets_names:
+                    raise ValueError('targets_to_learn keys must be dataset target variables')
+                var_id = targets_names.index(key)
+                for val in values:
+                    if val not in dataset.targets[var_id][1]:
+                        raise ValueError('targets_to_learn values must be in target variable domain')
+
+        feature_domains = dataset.features
+        target_domains = dataset.targets
         rules = []
-        #nb_variables = len(data[0][1])
 
-        # Extract observed values
-        #values = []
-        #for var in range(0, nb_variables):
-        #    v = []
+        # Replace state variable value (string) by their domain id (int)
+        encoded_data = Algorithm.encode_transitions_set(dataset.data, dataset.features, dataset.targets)
 
-        #    for t1, t2 in transitions:
-        #        if t2[var] not in v:
-        #            v.append(t2[var])
-
-        #    values.append(v)
-
-        #print("Set of values: "+str(values))
+        # Nothing to learn
+        if len(encoded_data) == 0:
+            return []
 
         # Learn rules for each observed variable/value
-        for var in range(0, len(target_domains)):
-            for val in range(0, len(target_domains[var][1])):
-                positives, negatives = PRIDE.interprete(data, var, val)
-                rules += PRIDE.fit_var_val(var, val, len(feature_domains), positives, negatives)
+        #for var in range(0, len(target_domains)):
+        #    for val in range(0, len(target_domains[var][1])):
+        for var_id, (var_name, var_domain) in enumerate(dataset.targets):
+            #eprint(var_id, (var_name, var_domain))
+            for val_id, val_name in enumerate(var_domain):
+                #eprint(val_id, val_name)
+                if var_name not in targets_to_learn:
+                    continue
+                if val_name not in targets_to_learn[var_name]:
+                    continue
+                positives, negatives = PRIDE.interprete(encoded_data, var_id, val_id)
+                rules += PRIDE.fit_var_val(var_id, val_id, len(feature_domains), positives, negatives)
 
-        # Instanciate output logic program
-        #variables = [var for var in range(nb_variables)]
-        output = LogicProgram(feature_domains, target_domains, rules)
+        output = rules
 
         return output
 
@@ -97,7 +115,7 @@ class PRIDE (Algorithm):
 
 
     @staticmethod
-    def fit_var_val(variable, value, nb_variables, positives, negatives):
+    def fit_var_val(variable, value, nb_features, positives, negatives):
         """
         Learn minimal rules that explain positive examples while consistent with negatives examples
 
@@ -123,7 +141,7 @@ class PRIDE (Algorithm):
             target = remaining[0]
             #eprint("new target: "+str(target))
 
-            R = Rule(variable, value, nb_variables)
+            R = Rule(variable, value, nb_features)
             #eprint(R.to_string())
 
             # 1) Consistency: against negatives examples
